/* Keyboard controlled test module for Lunar Lander AI challenge.
 * Brad Smith 2/07/2012 (rainwarrior@gmail.com)
 *
 * Press cursor keys left/right to rotate, up to thrust.
 */

// for ::GetKeyState
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

// for ::sqrtf
#include <cmath>

#include "ai_interface.h"

const unsigned int COLOR_LOCATION = 0xFFFFFF00;
const unsigned int COLOR_VELOCITY = 0xFF00FF00;
const unsigned int COLOR_SENSOR   = 0xFF0000FF;
const unsigned int COLOR_CLOUD    = 0xFF554433;

namespace Lunar_AI
{
    const char * Get_AI_Name()
    {
        return "keyboard";
    }

    void Init()
    {
    }

    void Cleanup()
    {
    }

    void Tick( float dt, float x, float y, float vx, float vy, float rot,
                float gx, float gy,
                float sense[ NUM_SENSORS ], float fuel,
                bool &out_thrust, bool &out_left, bool &out_right)
    {
        float velocity = ::sqrtf((vx * vx) + (vy * vy));

        Draw_Text(x - 2.0f, y - 2.0f, 0.5f, false, COLOR_LOCATION, "(%6.2f, %6.2f)", x, y);
        Draw_Text(x - 2.0f, y - 2.5f, 0.5f, false, COLOR_VELOCITY, "(%6.2f, %6.2f) %6.2f", vx, vy, velocity);
        Draw_Line(x, y, x+vx, y+vy, false, COLOR_VELOCITY);

        const float RT2O2 = ::sqrtf(2.0f) / 2.0f;

        // normalized direction vectors for sensors
        static const float SDIR[NUM_SENSORS][2] = {
            { -RT2O2,  RT2O2 },
            {   0.0f,   1.0f },
            {  RT2O2,  RT2O2 },
            {  -1.0f,   0.0f },
            {   1.0f,   0.0f },
            { -RT2O2, -RT2O2 },
            {   0.0f,  -1.0f },
            {  RT2O2, -RT2O2 } };

        for (int i=0; i < NUM_SENSORS; ++i)
        {
            float sx = x + (SDIR[i][0] * sense[i]);
            float sy = y + (SDIR[i][1] * sense[i]);
            Draw_Line(x, y, sx, sy, false, COLOR_SENSOR);
        }

        out_thrust = ::GetKeyState(VK_UP) < 0;
        out_left   = ::GetKeyState(VK_LEFT) < 0;
        out_right  = ::GetKeyState(VK_RIGHT) < 0;
    }

}// namespace Lunar_AI
